Imports System.Drawing.Drawing2D
Imports System.Xml
Public Class BackgroundPic
    Private m_Color As Color = Color.Blue
    Private m_HorizontalPics As Byte = 6
    Private m_VerticalPics As Byte = 4
    Private m_PicWidth As Integer = 80
    Private m_PicHeight As Integer = 60

    Private m_PicDistance As Integer = 10
    Private m_Kreisdurchmesser As Integer = 40
    Private m_UseBaseColor As Boolean = False

    Private m_Elements As PicElementCollection

    Public Event Changed()
    Private m_BMP As Bitmap


    Private m_FileName As String = ""

    ReadOnly Property Bitmap() As Bitmap
        Get
            Return m_BMP
        End Get
    End Property
    Property FileName() As String
        Get
            Return m_FileName
        End Get
        Set(ByVal value As String)
            m_FileName = value
        End Set
    End Property
    Public Sub New()
        CreateNewCollection()
        Recalculate_Bitmap()
    End Sub
    Public Sub New(ByVal FileName As String)
        m_FileName = FileName

        Dim Nd As XmlNode = ColorFunctions.GetXML(m_FileName)
        m_Color = Color.FromArgb(CByte(Nd.Attributes("ColR").Value), CByte(Nd.Attributes("ColG").Value), CByte(Nd.Attributes("ColB").Value))

        m_HorizontalPics = CByte(Nd.Attributes("HorizontalPics").Value)
        m_VerticalPics = CByte(Nd.Attributes("VerticalPics").Value)
        m_PicWidth = CInt(Nd.Attributes("PicWidth").Value)
        m_PicHeight = CInt(Nd.Attributes("PicHeight").Value)
        m_PicDistance = CInt(Nd.Attributes("PicDistance").Value)
        m_Kreisdurchmesser = CInt(Nd.Attributes("Kreisdurchmesser").Value)
        m_UseBaseColor = CBool(Nd.Attributes("UseBaseColor").Value)

        Dim CollNd As XmlNode = Nothing
        For Each xNd As XmlNode In Nd.ChildNodes
            If xNd.Name = "Collection" Then
                CollNd = xNd
                Exit For
            End If
        Next
        m_Elements = New PicElementCollection(CollNd)
        Recalculate_Bitmap()
    End Sub
    Public Sub Speichern()
        Dim Writer As XmlTextWriter = ColorFunctions.GetWriter(m_FileName)
        Write_XML(Writer)
        ColorFunctions.CloseWriter(Writer)
    End Sub
    Private Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteAttributeString("ColR", m_Color.R.ToString)
        Writer.WriteAttributeString("ColG", m_Color.G.ToString)
        Writer.WriteAttributeString("ColB", m_Color.B.ToString)

        Writer.WriteAttributeString("HorizontalPics", m_HorizontalPics.ToString)
        Writer.WriteAttributeString("VerticalPics", m_VerticalPics.ToString)
        Writer.WriteAttributeString("PicWidth", m_PicWidth.ToString)
        Writer.WriteAttributeString("PicHeight", m_PicHeight.ToString)
        Writer.WriteAttributeString("PicDistance", m_PicDistance.ToString)
        Writer.WriteAttributeString("Kreisdurchmesser", m_Kreisdurchmesser.ToString)
        Writer.WriteAttributeString("UseBaseColor", m_UseBaseColor.ToString)

        Writer.WriteStartElement("Collection")
        m_Elements.Write_XML(Writer)
        Writer.WriteEndElement()

    End Sub
    Property UseBaseColor() As Boolean
        Get
            Return m_UseBaseColor
        End Get
        Set(ByVal value As Boolean)
            If m_UseBaseColor <> value Then
                m_UseBaseColor = value
                Farben_Neu_Verteilen()
            End If
        End Set
    End Property
    Property PicDistance() As Integer
        Get
            Return m_PicDistance
        End Get
        Set(ByVal value As Integer)
            If m_PicDistance <> value Then
                m_PicDistance = value
                Recalculate_Bitmap()
            End If
        End Set
    End Property
    Property Radius() As Integer
        Get
            Return m_Kreisdurchmesser \ 2
        End Get
        Set(ByVal value As Integer)
            If m_Kreisdurchmesser <> value * 2 Then
                m_Kreisdurchmesser = value * 2
                Recalculate_Bitmap()
            End If
        End Set
    End Property
    Property Color() As Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As Color)
            If m_Color <> value Then
                m_Color = value
                If m_UseBaseColor Then
                    Farben_Neu_Verteilen()
                Else
                    Recalculate_Bitmap()
                End If
            End If
        End Set
    End Property
    Property HorizontalPics() As Byte
        Get
            Return m_HorizontalPics
        End Get
        Set(ByVal value As Byte)
            If m_HorizontalPics <> value Then
                m_HorizontalPics = value
                CreateNewCollection()
                Recalculate_Bitmap()
            End If
        End Set
    End Property
    Property VerticalPics() As Byte
        Get
            Return m_VerticalPics
        End Get
        Set(ByVal value As Byte)
            If m_VerticalPics <> value Then
                m_VerticalPics = value
                CreateNewCollection()
                Recalculate_Bitmap()
            End If
        End Set
    End Property
    Property PicWidth() As Integer
        Get
            Return m_PicWidth
        End Get
        Set(ByVal value As Integer)
            If m_PicWidth <> value Then
                m_PicWidth = value
                Recalculate_Bitmap()
            End If
        End Set
    End Property
    Property PicHeight() As Integer
        Get
            Return m_PicHeight
        End Get
        Set(ByVal value As Integer)
            If m_PicHeight <> value Then
                m_PicHeight = value
                Recalculate_Bitmap()
            End If
        End Set
    End Property

    Private Sub CreateNewCollection()
        If m_UseBaseColor Then
            m_Elements = New PicElementCollection(m_VerticalPics, m_HorizontalPics, m_Color)
        Else
            m_Elements = New PicElementCollection(m_VerticalPics, m_HorizontalPics, Color.Transparent)
        End If
    End Sub
    Public Sub Farben_Neu_Verteilen()
        If m_UseBaseColor Then
            m_Elements.FarbenNeu_Verteilen(m_Color)
        Else
            m_Elements.FarbenNeu_Verteilen(Color.Transparent)
        End If
        Recalculate_Bitmap()
    End Sub
 
    Private Sub Recalculate_Bitmap()
        Dim DrawElements As PicElementGroupCollection = m_Elements.DrawingElements

        Dim Width As Integer = (m_PicWidth + m_PicDistance) * CInt(m_HorizontalPics)
        Dim Height As Integer = (m_PicHeight + m_PicDistance) * CInt(m_VerticalPics)

        m_BMP = New Bitmap(Width, Height)

        Dim g As Graphics = Graphics.FromImage(m_BMP)
        g.Clear(m_Color)
        g.SmoothingMode = SmoothingMode.AntiAlias

        For Each Grp As PicElementGroup In DrawElements.RaisedElements
            DrawGroup(g, Grp)
        Next
        For Each Grp As PicElementGroup In DrawElements.SunkenElements
            DrawGroup(g, Grp)
        Next

        RaiseEvent Changed()
    End Sub

    Private Sub DrawGroup(ByVal g As Graphics, ByVal Grp As PicElementGroup)

        Dim Brsh As New SolidBrush(Grp.Color)
        Dim LightBrush As New SolidBrush(ControlPaint.Light(Grp.Color))
        Dim DarkBrush As New SolidBrush(ControlPaint.Dark(Grp.Color))

        Dim FullPath As New GraphicsPath
        FullPath.FillMode = FillMode.Winding
        For Each El As PicElement In Grp
            Dim Rect As Rectangle = GetBounds(El)


            If El.LUConnector Then
                FullPath.AddPath(TopLeftConnection(New Point(Rect.X, Rect.Y)), False)
            End If
            If El.RUConnector Then
                FullPath.AddPath(TopRightConnection(New Point(Rect.X, Rect.Y)), False)
            End If
            If El.LDConnector Then
                FullPath.AddPath(BottomLeftConnection(New Point(Rect.X, Rect.Y)), False)
            End If
            If El.RDConnector Then
                FullPath.AddPath(BottomRightConnection(New Point(Rect.X, Rect.Y)), False)
            End If

            If El.LineIndex = 0 Then
                If El.ColumnIndex = 0 Then
                    If El.LUConnector Then
                        Dim TmpRect As New Rectangle((m_PicWidth + m_PicDistance) * CInt(m_HorizontalPics) + m_PicDistance \ 2, 0 + m_PicDistance \ 2, m_PicWidth, m_PicHeight)
                        FullPath.AddPath(TopLeftConnection(New Point(TmpRect.X, TmpRect.Y)), False)
                        TmpRect = New Rectangle(0 + m_PicDistance \ 2, (m_PicHeight + m_PicDistance) * CInt(m_VerticalPics) + m_PicDistance \ 2, m_PicWidth, m_PicHeight)
                        FullPath.AddPath(TopLeftConnection(New Point(TmpRect.X, TmpRect.Y)), False)
                    End If
                ElseIf El.ColumnIndex = CInt(m_HorizontalPics - 1) Then
                    If El.RUConnector Then
                        Dim TmpRect As New Rectangle(m_PicDistance \ 2 - m_PicWidth - m_PicDistance, 0 + m_PicDistance \ 2, m_PicWidth, m_PicHeight)
                        FullPath.AddPath(TopRightConnection(New Point(TmpRect.X, TmpRect.Y)), False)
                        TmpRect = New Rectangle((m_PicWidth + m_PicDistance) * CInt(m_HorizontalPics - 1) + m_PicDistance \ 2, (m_PicHeight + m_PicDistance) * CInt(m_VerticalPics) + m_PicDistance \ 2, m_PicWidth, m_PicHeight)
                        FullPath.AddPath(TopRightConnection(New Point(TmpRect.X, TmpRect.Y)), False)
                    End If
                End If
            End If

            Dim GP As GraphicsPath = RoundRect(Rect, El)
            FullPath.AddPath(GP, False)
        Next

        g.TranslateTransform(-1, -1)
        g.FillPath(LightBrush, FullPath)
        g.TranslateTransform(2, 2)
        g.FillPath(DarkBrush, FullPath)
        g.TranslateTransform(-1, -1)
        g.FillPath(Brsh, FullPath)


        'Dim Pn As New Pen(Color.Gold)
        ''FullPath.Widen(Pn)

        'FullPath.Reverse()
        'g.DrawPath(Pn, FullPath)
        'Pn.Dispose()
        Brsh.Dispose()
        LightBrush.Dispose()
        DarkBrush.Dispose()
        'Dim LightBrush As New SolidBrush(ControlPaint.Light(Col))
        'Dim DarkBrush As New SolidBrush(ControlPaint.Dark(Col))
    End Sub




    Private Function TopLeftConnection(ByVal BasePoint As Point) As GraphicsPath
        Dim Path As New GraphicsPath

        Path.StartFigure()
        Path.AddLine(BasePoint.X, BasePoint.Y + m_Kreisdurchmesser, BasePoint.X, BasePoint.Y)
        Path.AddLine(BasePoint.X, BasePoint.Y, BasePoint.X + m_Kreisdurchmesser, BasePoint.Y)
        AddArc(Path, New PointF(BasePoint.X + m_Kreisdurchmesser, BasePoint.Y), m_Kreisdurchmesser + m_PicDistance, ArcDirections.LinksUnten)
        Path.AddLine(BasePoint.X - m_PicDistance, BasePoint.Y - m_Kreisdurchmesser - m_PicDistance, BasePoint.X - m_PicDistance, BasePoint.Y - m_PicDistance)
        Path.AddLine(BasePoint.X - m_PicDistance, BasePoint.Y - m_PicDistance, BasePoint.X - m_PicDistance - m_Kreisdurchmesser, BasePoint.Y - m_PicDistance)
        AddArc(Path, New PointF(BasePoint.X - m_Kreisdurchmesser - m_PicDistance, BasePoint.Y - m_PicDistance), m_Kreisdurchmesser + m_PicDistance, ArcDirections.RechtsOben)
        Path.CloseFigure()
        Return Path
    End Function

    Private Function TopRightConnection(ByVal BasePoint As Point) As GraphicsPath
        Dim Pt As New Point(BasePoint.X + m_PicWidth + m_PicDistance, BasePoint.Y)

        Dim Path As New GraphicsPath

        Path.StartFigure()
        AddArc(Path, New PointF(Pt.X - m_PicDistance, Pt.Y + m_Kreisdurchmesser), m_Kreisdurchmesser + m_PicDistance, ArcDirections.LinksOben)
        Path.AddLine(Pt.X + m_Kreisdurchmesser, Pt.Y - m_PicDistance, Pt.X, Pt.Y - m_PicDistance)
        Path.AddLine(Pt.X, Pt.Y - m_PicDistance, Pt.X, Pt.Y - m_PicDistance - m_Kreisdurchmesser)
        AddArc(Path, New PointF(Pt.X, Pt.Y - m_PicDistance - m_Kreisdurchmesser), m_Kreisdurchmesser + m_PicDistance, ArcDirections.RechtsUnten)
        Path.AddLine(Pt.X - m_PicDistance - m_Kreisdurchmesser, Pt.Y, Pt.X - m_PicDistance, Pt.Y)
        Path.AddLine(Pt.X - m_PicDistance, Pt.Y, Pt.X - m_PicDistance, Pt.Y + m_Kreisdurchmesser)

        Path.CloseFigure()
        Return Path
    End Function
    Private Function BottomRightConnection(ByVal BasePoint As Point) As GraphicsPath
        Dim Pt As New Point(BasePoint.X + m_PicWidth + m_PicDistance, BasePoint.Y + m_PicHeight + m_PicDistance)
        Return TopLeftConnection(Pt)
    End Function
    Private Function BottomLeftConnection(ByVal BasePoint As Point) As GraphicsPath
        Dim Pt As New Point(BasePoint.X - m_PicWidth - m_PicDistance, BasePoint.Y + m_PicHeight + m_PicDistance)
        Return TopRightConnection(Pt)
    End Function


    Private Function GetBounds(ByVal El As PicElement) As Rectangle
        Return New Rectangle((m_PicWidth + m_PicDistance) * CInt(El.ColumnIndex) + m_PicDistance \ 2, (m_PicHeight + m_PicDistance) * CInt(El.LineIndex) + m_PicDistance \ 2, m_PicWidth, m_PicHeight)
    End Function

    Private Function RoundRect(ByVal Rect As Rectangle, ByVal El As PicElement) As GraphicsPath
        Dim GP As New GraphicsPath

        Dim InnerRect As New Rectangle(Rect.X, Rect.Y, Rect.Width, Rect.Height)
        InnerRect.Inflate(-m_Kreisdurchmesser, -m_Kreisdurchmesser)

        Dim StartPoints(3) As PointF
        StartPoints(0) = New PointF(InnerRect.Right, Rect.Top)
        StartPoints(1) = New PointF(Rect.Right, InnerRect.Bottom)
        StartPoints(2) = New PointF(InnerRect.Left, Rect.Bottom)
        StartPoints(3) = New PointF(Rect.Left, InnerRect.Top)

        GP.StartFigure()

        'GP.AddLine(InnerRect.X, Rect.Y, InnerRect.Right, Rect.Y)
        If El.RUConnector Then
            GP.AddLine(InnerRect.Right, Rect.Y, Rect.Right, Rect.Y)
            GP.AddLine(Rect.Right, Rect.Y, Rect.Right, InnerRect.Y)
        Else
            AddArc(GP, StartPoints(0), m_Kreisdurchmesser, ArcDirections.RechtsOben)
        End If

        'GP.AddLine(Rect.Right, InnerRect.Top, Rect.Right, InnerRect.Bottom)
        If El.RDConnector Then
            GP.AddLine(Rect.Right, InnerRect.Bottom, Rect.Right, Rect.Bottom)
            GP.AddLine(Rect.Right, Rect.Bottom, InnerRect.Right, Rect.Bottom)
        Else
            AddArc(GP, StartPoints(1), m_Kreisdurchmesser, ArcDirections.RechtsUnten)
        End If

        'GP.AddLine(InnerRect.Right, Rect.Bottom, InnerRect.Left, Rect.Bottom)

        If El.LDConnector Then
            GP.AddLine(InnerRect.X, Rect.Bottom, Rect.X, Rect.Bottom)
            GP.AddLine(Rect.X, Rect.Bottom, Rect.X, InnerRect.Bottom)
        Else
            AddArc(GP, StartPoints(2), m_Kreisdurchmesser, ArcDirections.LinksUnten)
        End If

        'GP.AddLine(Rect.X, InnerRect.Bottom, Rect.X, InnerRect.Top)
        If El.LUConnector Then
            GP.AddLine(Rect.X, InnerRect.Y, Rect.X, Rect.Y)
            GP.AddLine(Rect.X, Rect.Y, InnerRect.X, Rect.Y)
        Else
            AddArc(GP, StartPoints(3), m_Kreisdurchmesser, ArcDirections.LinksOben)
        End If


        GP.CloseFigure()

        Return GP
    End Function

    Private Sub AddArc(ByVal GP As GraphicsPath, ByVal StartPoint As PointF, ByVal Diameter As Single, ByVal Direction As ArcDirections)
        If Direction = ArcDirections.RechtsOben Then
            'Bogen nach rechts unten
            GP.AddArc(StartPoint.X, StartPoint.Y, Diameter, Diameter, -90, 90)
        ElseIf Direction = ArcDirections.RechtsUnten Then
            'Bogen nach links unten
            GP.AddArc(StartPoint.X - Diameter, StartPoint.Y, Diameter, Diameter, 0, 90)
        ElseIf Direction = ArcDirections.LinksUnten Then
            'Bogen nach links oben
            GP.AddArc(StartPoint.X - Diameter, StartPoint.Y - Diameter, Diameter, Diameter, 90, 90)
        ElseIf Direction = ArcDirections.LinksOben Then
            'bogen nach rechts oben
            GP.AddArc(StartPoint.X, StartPoint.Y - Diameter, Diameter, Diameter, -180, 90)
        End If
    End Sub
    Public Sub Neu_Erstellen()
        CreateNewCollection()
        Recalculate_Bitmap()
    End Sub
    'Randomize
End Class
